#include <stdio.h>
#include <string.h>
#include <stdlib.h>

typedef struct {
  char ime[31], prezime[31];
  double godine;
  char zemlja[4];
  int tren_bodovi;
  int bodovi[52];
  int rang_zemlja;
} tenisac;

void osvjezi_tjedne(tenisac* t, int n, char** ime, char** prezime, int* bodovi, int m) {
  int i;
  /* Iteriramo po tenisacima koji su igrali posljednji tjedan. */
  for (i = 0; i < m; i++) {
    /* Koristimo binarno pretrazivanje kako bismo nasli trenutnog tenisaca na popisu svih koje pratimo. */ 
    int lo = 0, hi = n;
    while (lo < hi) {
      int mid = (lo + hi) / 2;
      int temp = strcmp(t[mid].prezime, prezime[i]);
      if (temp < 0 || (!temp && strcmp(t[mid].ime, ime[i]) < 0))
        lo = mid + 1;
      else
        hi = mid;
    }
    t[lo].bodovi[0] = bodovi[i];
  }
}

void azuriraj(tenisac* t, int n, char** ime, char** prezime, int* bodovi, int m) {
  int i, j, veci;
  /* Za svakog tenisaca izbacujemo njegove bodove osvojene prije 52 tjedna te novoosvojene bodove po defaultu postavljamo na nulu. */
  for (i = 0; i < n; i++) {
    veci = 0;
    for (j = 0; j < 51; j++)
      veci += t[i].bodovi[j] >= t[i].bodovi[51];
    if (veci < 19)
      t[i].tren_bodovi -= t[i].bodovi[51];
    for (j = 51; j; j--)
      t[i].bodovi[j] = t[i].bodovi[j - 1];
    t[i].bodovi[0] = 0;
  }
  osvjezi_tjedne(t, n, ime, prezime, bodovi, m);
  /* Za svakog tenisaca uracunavamo nove bodove. */
  for (i = 0; i < n; i++) {
    veci = 0;
    for (j = 1; j < 52; j++)
      veci += t[i].bodovi[j] > t[i].bodovi[0];
    if (veci < 19)
      t[i].tren_bodovi += t[i].bodovi[0];
  }
}

/*
Alternativno, racunanje novog trenutnog broja bodova i-tog tenisaca mozemo izvesti tako da 19 puta nadjemo maksimum u nizu t[i].bodovi
te ga izuzmemo iz daljnjeg razmatranja tako da mu negiramo vrijednost. Na kraju jos treba vratiti negirane vrijednosti na pocetne.
*/
void azuriraj_alt(tenisac* t, int n, char** ime, char** prezime, int* bodovi, int m) {
  int i, j, k;
  for (i = 0; i < n; i++) {
    for (j = 51; j; j--)
      t[i].bodovi[j] = t[i].bodovi[j - 1];
    t[i].bodovi[0] = 0;
  }
  osvjezi_tjedne(t, n, ime, prezime, bodovi, m);
  for (i = 0; i < n; i++) {
    t[i].tren_bodovi = 0;
    for (k = 0; k < 19; k++) {
      int poz = 0;
      for (j = 1; j < 52; j++)
        if (t[i].bodovi[j] > t[i].bodovi[poz])
          poz = j;
      t[i].tren_bodovi += t[i].bodovi[poz];
      t[i].bodovi[poz] *= -1;
    }
    for (j = 0; j < 52; j++)
      if (t[i].bodovi[j] < 0)
        t[i].bodovi[j] *= -1;
  }
}

void ispisi(tenisac* t, int n, int starost) {
  int i, j;
  for (i = 0; i < n - 1; i++)
    for (j = i + 1; j < n; j++)
      if (t[i].tren_bodovi < t[j].tren_bodovi) {
        tenisac temp = t[i];
        t[i] = t[j];
        t[j] = temp;
      }
  for (i = 0; i < n; i++)
    if (t[i].godine < starost) {
      t[i].rang_zemlja = 0;
      for (j = 0; j < i; j++)
        t[i].rang_zemlja += !strcmp(t[i].zemlja, t[j].zemlja);
      printf("%d %s %s %.1lf %s%d %d\n", i + 1, t[i].ime, t[i].prezime, t[i].godine, t[i].zemlja, t[i].rang_zemlja + 1, t[i].tren_bodovi);
    }
}

int cmp_zemlja(const void* t1, const void* t2) {
  return strcmp(((tenisac*)t1) -> zemlja, ((tenisac*)t2) -> zemlja);
}

int cmp_bodovi(const void* t1, const void* t2) {
  return ((tenisac*)t2) -> tren_bodovi - ((tenisac*)t1) -> tren_bodovi;
}

/* 
Za dodatne bodove u (c) dijelu, najprije sortiramo tenisace po zemlji. Tenisaci iz svake zemlje cine blok u tom nizu te 
svaki takav blok sortiramo silazno po bodovima. Na taj nacin odredimo rang svakog tenisaca unutar zemlje te ga zapamtimo
unutar strukture. Na kraju, sve tenisace sortiramo silazno po bodovima te ispisemo one koji su mladji od dane starosti.
*/
void ispisi_brze(tenisac* t, int n, int starost) {
  int i = 0, j, k; 
  qsort(t, n, sizeof(tenisac), cmp_zemlja);
  while (i < n) {
    j = i;
    while (j < n && !cmp_zemlja(t + i, t + j))
      j++;
    qsort(t + i, j - i, sizeof(tenisac), cmp_bodovi);
    for (k = i; k < j; k++)
      t[k].rang_zemlja = k - i;
    i = j;
  }
  qsort(t, n, sizeof(tenisac), cmp_bodovi);
  for (i = 0; i < n; i++)
    if (t[i].godine < starost)
      printf("%d %s %s %.1lf %s%d %d\n", i + 1, t[i].ime, t[i].prezime, t[i].godine, t[i].zemlja, t[i].rang_zemlja + 1, t[i].tren_bodovi);
}

int main() {
  tenisac t[4] = {{"Carlos", "Alcaraz", 20.1, "ESP", 13140, {300, 0, 360, 45, 45, 2000, 90, 500, 300, 90, 90, 0, 500, 0, 500, 45, 20, 2000, 600, 300, 0, 0, 500, 500, 10, 720, 300, 360, 0, 90, 500, 0, 1000, 500, 20, 180, 500, 250, 180, 45, 45, 10, 0, 500, 0, 1000, 180, 0, 300, 0, 45, 45}},
                  {"Novak", "Djokovic", 36, "SRB", 14100, {150, 180, 2000, 250, 250, 500, 500, 0, 10, 10, 10, 0, 1000, 0, 360, 500, 45, 500, 0, 500, 0, 1000, 1000, 500, 0, 500, 600, 500, 500, 90, 0, 0, 0, 0, 45, 0, 180, 1000, 500, 45, 0, 500, 0, 1000, 360, 500, 90, 1000, 500, 500, 0, 0}},
                  {"Daniil", "Medvedev", 27.3, "RUS", 12740, {500, 1000, 45, 0, 360, 500, 0, 1000, 90, 20, 180, 90, 250, 0, 45, 0, 45, 0, 150, 2000, 180, 180, 2000, 0, 10, 250, 180, 180, 300, 45, 180, 500, 0, 0, 45, 0, 300, 500, 45, 0, 0, 0, 1000, 600, 360, 90, 360, 360, 150, 45, 600, 0}},
                  {"Rafael", "Nadal", 37, "ESP", 11205, {0, 1000, 0, 0, 2000, 0, 0, 250, 0, 0, 0, 0, 180, 0, 45, 20, 20, 2000, 2000, 0, 0, 0, 45, 500, 0, 250, 0, 180, 180, 0, 0, 0, 0, 0, 180, 90, 250, 500, 0, 0, 1000, 0, 0, 360, 0, 0, 0, 0, 90, 0, 0, 150}}};
  char* imena[] = {"Novak", "Daniil", "Carlos"};
  char* prezimena[] = {"Djokovic", "Medvedev", "Alcaraz"};
  int bodovi[] = {2000, 0, 720};
  azuriraj(t, 4, imena, prezimena, bodovi, 3);
  ispisi(t, 4, 40);
  return 0;
}
