#include <stdio.h>
#include <string.h>
#include <stdlib.h>

typedef struct {
  char ime[31], prezime[31];
  double godine;
  char zemlja[4];
  int tren_bodovi;
  int bodovi[52];
  int rang_zemlja;
} tenisacica;

void osvjezi_tjedne(tenisacica* t, int n, char** ime, char** prezime, int* bodovi, int m) {
  int i;
  /* Iteriramo po tenisacicama koje su igrale posljednji tjedan. */
  for (i = 0; i < m; i++) {
    /* Koristimo binarno pretrazivanje kako bismo nasli trenutnu tenisacicu na popisu svih koje pratimo. */ 
    int lo = 0, hi = n;
    while (lo < hi) {
      int mid = (lo + hi) / 2;
      int temp = strcmp(t[mid].prezime, prezime[i]);
      if (temp < 0 || (!temp && strcmp(t[mid].ime, ime[i]) < 0))
        lo = mid + 1;
      else
        hi = mid;
    }
    t[lo].bodovi[0] = bodovi[i];
  }
}

void azuriraj(tenisacica* t, int n, char** ime, char** prezime, int* bodovi, int m) {
  int i, j, veci;
  /* Za svaku tenisacicu izbacujemo njene bodove osvojene prije 52 tjedna te novoosvojene bodove po defaultu postavljamo na nulu. */
  for (i = 0; i < n; i++) {
    veci = 0;
    for (j = 0; j < 51; j++)
      veci += t[i].bodovi[j] >= t[i].bodovi[51];
    if (veci < 16)
      t[i].tren_bodovi -= t[i].bodovi[51];
    for (j = 51; j; j--)
      t[i].bodovi[j] = t[i].bodovi[j - 1];
    t[i].bodovi[0] = 0;
  }
  osvjezi_tjedne(t, n, ime, prezime, bodovi, m);
  /* Za svaku tenisacicu uracunavamo nove bodove. */
  for (i = 0; i < n; i++) {
    veci = 0;
    for (j = 1; j < 52; j++)
      veci += t[i].bodovi[j] > t[i].bodovi[0];
    if (veci < 16)
      t[i].tren_bodovi += t[i].bodovi[0];
  }
}

/*
Alternativno, racunanje novog trenutnog broja bodova i-te tenisacice mozemo izvesti tako da 16 puta nadjemo maksimum u nizu t[i].bodovi
te ga izuzmemo iz daljnjeg razmatranja tako da mu negiramo vrijednost. Na kraju jos treba vratiti negirane vrijednosti na pocetne.
*/
void azuriraj_alt(tenisacica* t, int n, char** ime, char** prezime, int* bodovi, int m) {
  int i, j, k;
  for (i = 0; i < n; i++) {
    for (j = 51; j; j--)
      t[i].bodovi[j] = t[i].bodovi[j - 1];
    t[i].bodovi[0] = 0;
  }
  osvjezi_tjedne(t, n, ime, prezime, bodovi, m);
  for (i = 0; i < n; i++) {
    t[i].tren_bodovi = 0;
    for (k = 0; k < 16; k++) {
      int poz = 0;
      for (j = 1; j < 52; j++)
        if (t[i].bodovi[j] > t[i].bodovi[poz])
          poz = j;
      t[i].tren_bodovi += t[i].bodovi[poz];
      t[i].bodovi[poz] *= -1;
    }
    for (j = 0; j < 52; j++)
      if (t[i].bodovi[j] < 0)
        t[i].bodovi[j] *= -1;
  }
}

void ispisi(tenisacica* t, int n, int starost) {
  int i, j;
  for (i = 0; i < n - 1; i++)
    for (j = i + 1; j < n; j++)
      if (t[i].tren_bodovi < t[j].tren_bodovi) {
        tenisacica temp = t[i];
        t[i] = t[j];
        t[j] = temp;
      }
  for (i = 0; i < n; i++)
    if (t[i].godine < starost) {
      t[i].rang_zemlja = 0;
      for (j = 0; j < i; j++)
        t[i].rang_zemlja += !strcmp(t[i].zemlja, t[j].zemlja);
      printf("%d %s %s %.1lf %s%d %d\n", i + 1, t[i].ime, t[i].prezime, t[i].godine, t[i].zemlja, t[i].rang_zemlja + 1, t[i].tren_bodovi);
    }
}

int cmp_zemlja(const void* t1, const void* t2) {
  return strcmp(((tenisacica*)t1) -> zemlja, ((tenisacica*)t2) -> zemlja);
}

int cmp_bodovi(const void* t1, const void* t2) {
  return ((tenisacica*)t2) -> tren_bodovi - ((tenisacica*)t1) -> tren_bodovi;
}

/* 
Za dodatne bodove u (c) dijelu, najprije sortiramo tenisacice po zemlji. Tenisacice iz svake zemlje cine blok u tom nizu te 
svaki takav blok sortiramo silazno po bodovima. Na taj nacin odredimo rang svake tenisacice unutar zemlje te ga zapamtimo
unutar strukture. Na kraju, sve tenisacice sortiramo silazno po bodovima te ispisemo one koje su mladje od dane starosti.
*/
void ispisi_brze(tenisacica* t, int n, int starost) {
  int i = 0, j, k; 
  qsort(t, n, sizeof(tenisacica), cmp_zemlja);
  while (i < n) {
    j = i;
    while (j < n && !cmp_zemlja(t + i, t + j))
      j++;
    qsort(t + i, j - i, sizeof(tenisacica), cmp_bodovi);
    for (k = i; k < j; k++)
      t[k].rang_zemlja = k - i;
    i = j;
  }
  qsort(t, n, sizeof(tenisacica), cmp_bodovi);
  for (i = 0; i < n; i++)
    if (t[i].godine < starost)
      printf("%d %s %s %.1lf %s%d %d\n", i + 1, t[i].ime, t[i].prezime, t[i].godine, t[i].zemlja, t[i].rang_zemlja + 1, t[i].tren_bodovi);
}

int main() {
  tenisacica t[4] = {{"Coco", "Gauff", 19.2, "USA", 7885, {0, 215, 70, 185, 55, 1000, 130, 0, 0, 120, 120, 1, 120, 60, 120, 120, 70, 100, 470, 390, 215, 305, 470, 0, 1000, 70, 0, 650, 0, 1000, 10, 305, 215, 305, 130, 185, 110, 1, 0, 470, 0, 110, 305, 100, 215, 185, 185, 390, 1, 305, 305, 0}},
                  {"Jessica", "Pegula", 29.3, "USA", 7765, {780, 0, 0, 120, 215, 100, 100, 240, 280, 0, 1300, 0, 0, 0, 0, 120, 10, 0, 390, 470, 120, 0, 110, 60, 180, 305, 185, 60, 110, 0, 180, 305, 650, 0, 390, 470, 280, 470, 0, 305, 430, 70, 0, 1, 215, 470, 0, 0, 0, 470, 0, 0}},
                  {"Aryna", "Sabalenka", 25.1, "BLR", 10825, {0, 215, 70, 280, 0, 0, 305, 280, 1000, 0, 10, 390, 470, 30, 470, 110, 650, 390, 1, 180, 55, 215, 100, 305, 2000, 180, 10, 0, 0, 1000, 55, 1300, 60, 305, 650, 100, 280, 305, 0, 180, 305, 100, 650, 55, 10, 470, 0, 110, 0, 280, 0, 470}},
                  {"Iga", "Swiatek", 22, "POL", 13550, {120, 305, 65, 470, 0, 120, 650, 185, 185, 1000, 1000, 0, 2000, 470, 65, 470, 185, 470, 305, 470, 30, 0, 0, 1000, 0, 1300, 280, 780, 470, 10, 0, 470, 215, 10, 0, 1000, 470, 0, 1000, 0, 60, 0, 215, 0, 470, 305, 215, 100, 470, 1000, 470, 55}}};
  char* imena[] = {"Iga", "Coco", "Jessica"};
  char* prezimena[] = {"Swiatek", "Gauff", "Pegula"};
  int bodovi[] = {130, 780, 240};
  azuriraj(t, 4, imena, prezimena, bodovi, 3);
  ispisi(t, 4, 30);
  return 0;
}
